/*
** Module   : SaxWriter
** Abstract : SAX writer specific interface.
**
** Copyright (c) 2013, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- ---------------------------------Description----------------------------------
** 001 EVL 20130208 Created initial version.
** 002 EVL 20130320 Adding SAX writer related literal constants.
** 003 EVK 20130903 Moved encoding methods to the new interface Encoding.
*/

package com.goldencode.p2j.xml;

import com.goldencode.p2j.util.*;

/**
 * Interface describing SAX-Writer specific features of the SAX objects.
 */
public interface SaxWriter
extends SaxEntity,
        Encoding
{
   /**
    * The SAX-WRITE-IDLE constant returned by the PARSE-STATUS attribute, which in 4GL is the
    * integer value 1.
    */
   public final static int SAX_WRITE_IDLE = 1;

   /**
    * The SAX-WRITE-BEGIN constant returned by the PARSE-STATUS attribute, which in 4GL is the
    * integer value 2.
    */
   public final static int SAX_WRITE_BEGIN = 2;

   /**
    * The SAX-WRITE-TAG constant returned by the PARSE-STATUS attribute, which in 4GL is the
    * integer value 3.
    */
   public final static int SAX_WRITE_TAG = 3;

   /**
    * The SAX-WRITE-ELEMENT constant returned by the PARSE-STATUS attribute, which in 4GL is the
    * integer value 4.
    */
   public final static int SAX_WRITE_ELEMENT = 4;

   /**
    * The SAX-WRITE-CONTENT constant returned by the PARSE-STATUS attribute, which in 4GL is the
    * integer value 5.
    */
   public final static int SAX_WRITE_CONTENT = 5;

   /**
    * The SAX-WRITE-COMPLETE constant returned by the PARSE-STATUS attribute, which in 4GL is the
    * integer value 6.
    */
   public final static int SAX_WRITE_COMPLETE = 6;

   /**
    * The SAX-WRITE-ERROR constant returned by the PARSE-STATUS attribute, which in 4GL is the
    * integer value 7.
    */
   public final static int SAX_WRITE_ERROR = 7;

   /**
    * Gets the value of the formatted attribute for the given SAX-Writer object. Formatted means
    * extra space or carrage returns should be added to preserve the hierarchical view of the
    * document.
    * 
    * @return   The <code>true</code> in case of formatted <code>false</code> otherwise.
    */
   public logical getFormatted();
   
   /**
    * Sets the value of the formatted attribute for the given SAX-Writer object. Formatted means
    * extra space or carrage returns should be added to preserve the hierarchical view of the
    * document.
    * 
    * @param    isFormatted
    *           The <code>true</code> in case of formatted <code>false</code> otherwise.
    */
   public void setFormatted(logical isFormatted);

   /**
    * Sets the value of the formatted attribute for the given SAX-Writer object. Formatted means
    * extra space or carrage returns should be added to preserve the hierarchical view of the
    * document.
    * 
    * @param    isFormatted
    *           The <code>true</code> in case of formatted <code>false</code> otherwise.
    */
   public void setFormatted(boolean isFormatted);

   /**
    * Gets the value of the strict attribute for the given SAX-Writer object. Strict means
    * SAX-Writer should make well formed XML document. The default is true.
    * 
    * @return   The <code>true</code> in case of strict <code>false</code> otherwise.
    */
   public logical getStrict();
   
   /**
    * Sets the value of the strict attribute for the given SAX-Writer object. Strict means
    * SAX-Writer should make well formed XML document. The default is true.
    * 
    * @param    isStrict
    *           The <code>true</code> in case of strict <code>false</code> otherwise.
    */
   public void setStrict(logical isStrict);

   /**
    * Sets the value of the strict attribute for the given SAX-Writer object. Strict means
    * SAX-Writer should make well formed XML document. The default is true.
    * 
    * @param    isStrict
    *           The <code>true</code> in case of strict <code>false</code> otherwise.
    */
   public void setStrict(boolean isStrict);

   /**
    * Gets the value of the fragment attribute for the given SAX-Writer object. Fragment means
    * SAX-Writer output is part of the document or full.
    * 
    * @return   The <code>true</code> in case of fragment <code>false</code> for full document.
    */
   public logical getFragment();
   
   /**
    * Sets the value of the fragment attribute for the given SAX-Writer object. Fragment means
    * SAX-Writer output is part of the document or full.
    * 
    * @param    isFragment
    *           The <code>true</code> in case of fragment <code>false</code> for full document.
    */
   public void setFragment(logical isFragment);
   
   /**
    * Sets the value of the fragment attribute for the given SAX-Writer object. Fragment means
    * SAX-Writer output is part of the document or full.
    * 
    * @param    isFragment
    *           The <code>true</code> in case of fragment <code>false</code> for full document.
    */
   public void setFragment(boolean isFragment);
   
   /**
    * Gets the value of the standalone attribute for the given SAX-Writer object.
    * 
    * @return   The character value of the standalone attribute.
    */
   public logical getStandalone();
   
   /**
    * Sets the value of the standalone attribute for the given SAX-Writer object.
    * 
    * @param    isStandalone
    *           The <code>true</code> in case of standalone <code>false</code> otherwise.
    */
   public void setStandalone(logical isStandalone);
   
   /**
    * Sets the value of the standalone attribute for the given SAX-Writer object.
    * 
    * @param    isStandalone
    *           The <code>true</code> in case of standalone <code>false</code> otherwise.
    */
   public void setStandalone(boolean isStandalone);
   
   /**
    * Gets the value of the version attribute for the given SAX-Writer object.
    * 
    * @return   The character value of the version attribute.
    */
   public character getVersion();
   
   /**
    * Sets the new value of the version attribute for the given SAX-Writer object.
    * 
    * @param    version
    *           The version value to be set for the given writer.
    */
   public void setVersion(String version);
   
   /**
    * Sets the new value of the version attribute for the given SAX-Writer object.
    * 
    * @param    version
    *           The version value to be set for the given writer.
    */
   public void setVersion(character version);
   
   /**
    * Gets the value of the write-status attribute for the given SAX-Writer object. This is one
    * of the SAX_WRITE_* constants defined by the {@link SaxWriter} interface.
    * <p>
    * Documentation states that the type of this attribute is character, but this is incorrect. 
    * The attribute's real type is integer, same as PARSE-STATUS for the SAX-Reader object.
    * 
    * @return   The integer value of the write-status attribute.
    */
   public integer getWriteStatus();
   
   /**
    * Adds namespace declaration to a tag in a XML document associated with SAX-Writer object.
    * 
    * @param    uri
    *           The URI namespace to add.
    * @param    prefix
    *           The prefix of the namespace to add.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical declareNamespace(Text uri, Text prefix);

   /**
    * Adds namespace declaration to a tag in a XML document associated with SAX-Writer object.
    * 
    * @param    uri
    *           The URI namespace to add.
    * @param    prefix
    *           The prefix of the namespace to add.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical declareNamespace(String uri, Text prefix);

   /**
    * Adds namespace declaration to a tag in a XML document associated with SAX-Writer object.
    * 
    * @param    uri
    *           The URI namespace to add.
    * @param    prefix
    *           The prefix of the namespace to add.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical declareNamespace(Text uri, String prefix);

   /**
    * Adds namespace declaration to a tag in a XML document associated with SAX-Writer object.
    * 
    * @param    uri
    *           The URI namespace to add.
    * @param    prefix
    *           The prefix of the namespace to add.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical declareNamespace(String uri, String prefix);

   /**
    * Adds namespace declaration to a tag in a XML document associated with SAX-Writer object.
    * 
    * @param    uri
    *           The URI namespace to add.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical declareNamespace(String uri);
   
   /**
    * Adds namespace declaration to a tag in a XML document associated with SAX-Writer object.
    * 
    * @param    uri
    *           The URI namespace to add.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical declareNamespace(Text uri);
   
   /**
    * Closes the XML document associated with SAX-Writer object.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical endDocument();

   /**
    * Ends an XML node of the given name in a XML document associated with SAX-Writer object.
    * 
    * @param    name
    *           The name of the node to end.
    * @param    uri
    *           The URI namespace of the node to end.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical endElement(Text name, Text uri);

   /**
    * Ends an XML node of the given name in a XML document associated with SAX-Writer object.
    * 
    * @param    name
    *           The name of the node to end.
    * @param    uri
    *           The URI namespace of the node to end.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical endElement(String name, Text uri);

   /**
    * Ends an XML node of the given name in a XML document associated with SAX-Writer object.
    * 
    * @param    name
    *           The name of the node to end.
    * @param    uri
    *           The URI namespace of the node to end.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical endElement(Text name, String uri);

   /**
    * Ends an XML node of the given name in a XML document associated with SAX-Writer object.
    * 
    * @param    name
    *           The name of the node to end.
    * @param    uri
    *           The URI namespace of the node to end.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical endElement(String name, String uri);

   /**
    * Ends an XML node of the given name in a XML document associated with SAX-Writer object.
    * 
    * @param    name
    *           The name of the node to end.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical endElement(String name);
   
   /**
    * Ends an XML node of the given name in a XML document associated with SAX-Writer object.
    * 
    * @param    name
    *           The name of the node to end.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical endElement(Text name);
   
   /**
    * Specifies the output destination of the XML document to be created by a SAX-Writer object.
    * 
    * @param    mode
    *           The type of the destination to write to. Can be 'FILE', 'STREAM', 'MEMPTR', 
    *           'STREAM-HANDLE' or 'LONGCHAR'.
    * @param    destination
    *           The appropriate object to write.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical setOutputDestination(String mode, Object destination);

   /**
    * Specifies the output destination of the XML document to be created by a SAX-Writer object.
    * 
    * @param    mode
    *           The type of the destination to write to. Can be 'FILE', 'STREAM', 'MEMPTR', 
    *           'STREAM-HANDLE' or 'LONGCHAR'.
    * @param    destination
    *           The appropriate object to write.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical setOutputDestination(character mode, Object destination);

   /**
    * Creates the XML document with the prolog information.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical startDocument();

   /**
    * Starts the XML node according to the specified name, namespace URI and handle of the SAX
    * attribute object.
    * 
    * @param    name
    *           The name of the element.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical startElement(Text name);

   /**
    * Starts the XML node according to the specified name, namespace URI and handle of the SAX
    * attribute object.
    * 
    * @param    name
    *           The name of the element.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical startElement(String name);

   /**
    * Starts the XML node according to the specified name, namespace URI and handle of the SAX
    * attribute object.
    * 
    * @param    name
    *           The name of the element.
    * @param    handle
    *           The handle of the SAX-Attribute object to be added into XML document.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical startElement(Text name, handle handle);

   /**
    * Starts the XML node according to the specified name, namespace URI and handle of the SAX
    * attribute object.
    * 
    * @param    name
    *           The name of the element.
    * @param    handle
    *           The handle of the SAX-Attribute object to be added into XML document.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical startElement(String name, handle handle);

   /**
    * Starts the XML node according to the specified name, namespace URI and handle of the SAX
    * attribute object.
    * 
    * @param    name
    *           The name of the element.
    * @param    uri
    *           The namespace URI of the given element.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical startElement(Text name, Text uri);

   /**
    * Starts the XML node according to the specified name, namespace URI and handle of the SAX
    * attribute object.
    * 
    * @param    name
    *           The name of the element.
    * @param    uri
    *           The namespace URI of the given element.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical startElement(String name, Text uri);

   /**
    * Starts the XML node according to the specified name, namespace URI and handle of the SAX
    * attribute object.
    * 
    * @param    name
    *           The name of the element.
    * @param    uri
    *           The namespace URI of the given element.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical startElement(Text name, String uri);

   /**
    * Starts the XML node according to the specified name, namespace URI and handle of the SAX
    * attribute object.
    * 
    * @param    name
    *           The name of the element.
    * @param    uri
    *           The namespace URI of the given element.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical startElement(String name, String uri);

   /**
    * Starts the XML node according to the specified name, namespace URI and handle of the SAX
    * attribute object.
    * 
    * @param    name
    *           The name of the element.
    * @param    uri
    *           The namespace URI of the given element.
    * @param    handle
    *           The handle of the SAX-Attribute object to be added into XML document.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical startElement(String name, Text uri, handle handle);

   /**
    * Starts the XML node according to the specified name, namespace URI and handle of the SAX
    * attribute object.
    * 
    * @param    name
    *           The name of the element.
    * @param    uri
    *           The namespace URI of the given element.
    * @param    handle
    *           The handle of the SAX-Attribute object to be added into XML document.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical startElement(Text name, String uri, handle handle);

   /**
    * Starts the XML node according to the specified name, namespace URI and handle of the SAX
    * attribute object.
    * 
    * @param    name
    *           The name of the element.
    * @param    uri
    *           The namespace URI of the given element.
    * @param    handle
    *           The handle of the SAX-Attribute object to be added into XML document.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical startElement(String name, String uri, handle handle);

   /**
    * Starts the XML node according to the specified name, namespace URI and handle of the SAX
    * attribute object.
    * 
    * @param    name
    *           The name of the element.
    * @param    uri
    *           The namespace URI of the given element.
    * @param    handle
    *           The handle of the SAX-Attribute object to be added into XML document.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical startElement(Text name, Text uri, handle handle);

   /**
    * Adds CDATA block to the XML document crating by SAX-Writer object.
    * 
    * @param    cdataBlock
    *           The data to be written with CDATA block.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical writeCdata(Text cdataBlock);

   /**
    * Adds CDATA block to the XML document crating by SAX-Writer object.
    * 
    * @param    cdataBlock
    *           The data to be written with CDATA block.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical writeCdata(String cdataBlock);

   /**
    * Adds character data to the XML document crating by SAX-Writer object.
    * 
    * @param    chardata
    *           The character or longchar data to be written.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical writeCharacters(Text chardata);

   /**
    * Adds character data to the XML document crating by SAX-Writer object.
    * 
    * @param    chardata
    *           The character or longchar data to be written.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical writeCharacters(String chardata);

   /**
    * Creates complete XML node with a given SAX-Writer object.
    * 
    * @param    name
    *           The name of the element.
    * @param    chardata
    *           The XML text to write for the given element.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical writeDataElement(Text name, Text chardata);

   /**
    * Creates complete XML node with a given SAX-Writer object.
    * 
    * @param    name
    *           The name of the element.
    * @param    chardata
    *           The XML text to write for the given element.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical writeDataElement(String name, Text chardata);

   /**
    * Creates complete XML node with a given SAX-Writer object.
    * 
    * @param    name
    *           The name of the element.
    * @param    chardata
    *           The XML text to write for the given element.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical writeDataElement(Text name, String chardata);

   /**
    * Creates complete XML node with a given SAX-Writer object.
    * 
    * @param    name
    *           The name of the element.
    * @param    chardata
    *           The XML text to write for the given element.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical writeDataElement(String name, String chardata);

   /**
    * Creates complete XML node with a given SAX-Writer object.
    * 
    * @param    name
    *           The name of the element.
    * @param    chardata
    *           The XML text to write for the given element.
    * @param    uri
    *           The namespace URI of the given element.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical writeDataElement(Text name, Text chardata, Text uri);

   /**
    * Creates complete XML node with a given SAX-Writer object.
    * 
    * @param    name
    *           The name of the element.
    * @param    chardata
    *           The XML text to write for the given element.
    * @param    uri
    *           The namespace URI of the given element.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical writeDataElement(String name, Text chardata, Text uri);

   /**
    * Creates complete XML node with a given SAX-Writer object.
    * 
    * @param    name
    *           The name of the element.
    * @param    chardata
    *           The XML text to write for the given element.
    * @param    uri
    *           The namespace URI of the given element.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical writeDataElement(Text name, String chardata, Text uri);

   /**
    * Creates complete XML node with a given SAX-Writer object.
    * 
    * @param    name
    *           The name of the element.
    * @param    chardata
    *           The XML text to write for the given element.
    * @param    uri
    *           The namespace URI of the given element.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical writeDataElement(String name, String chardata, Text uri);

   /**
    * Creates complete XML node with a given SAX-Writer object.
    * 
    * @param    name
    *           The name of the element.
    * @param    chardata
    *           The XML text to write for the given element.
    * @param    uri
    *           The namespace URI of the given element.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical writeDataElement(Text name, Text chardata, String uri);

   /**
    * Creates complete XML node with a given SAX-Writer object.
    * 
    * @param    name
    *           The name of the element.
    * @param    chardata
    *           The XML text to write for the given element.
    * @param    uri
    *           The namespace URI of the given element.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical writeDataElement(String name, Text chardata, String uri);

   /**
    * Creates complete XML node with a given SAX-Writer object.
    * 
    * @param    name
    *           The name of the element.
    * @param    chardata
    *           The XML text to write for the given element.
    * @param    uri
    *           The namespace URI of the given element.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical writeDataElement(Text name, String chardata, String uri);

   /**
    * Creates complete XML node with a given SAX-Writer object.
    * 
    * @param    name
    *           The name of the element.
    * @param    chardata
    *           The XML text to write for the given element.
    * @param    uri
    *           The namespace URI of the given element.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical writeDataElement(String name, String chardata, String uri);

   /**
    * Creates complete XML node with a given SAX-Writer object.
    * 
    * @param    name
    *           The name of the element.
    * @param    chardata
    *           The XML text to write for the given element.
    * @param    handle
    *           The handle of the SAX-Attribute object to be added into XML document.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical writeDataElement(Text name, Text chardata, handle handle);

   /**
    * Creates complete XML node with a given SAX-Writer object.
    * 
    * @param    name
    *           The name of the element.
    * @param    chardata
    *           The XML text to write for the given element.
    * @param    handle
    *           The handle of the SAX-Attribute object to be added into XML document.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical writeDataElement(String name, Text chardata, handle handle);

   /**
    * Creates complete XML node with a given SAX-Writer object.
    * 
    * @param    name
    *           The name of the element.
    * @param    chardata
    *           The XML text to write for the given element.
    * @param    handle
    *           The handle of the SAX-Attribute object to be added into XML document.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical writeDataElement(Text name, String chardata, handle handle);

   /**
    * Creates complete XML node with a given SAX-Writer object.
    * 
    * @param    name
    *           The name of the element.
    * @param    chardata
    *           The XML text to write for the given element.
    * @param    handle
    *           The handle of the SAX-Attribute object to be added into XML document.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical writeDataElement(String name, String chardata, handle handle);

   /**
    * Creates complete XML node with a given SAX-Writer object.
    * 
    * @param    name
    *           The name of the element.
    * @param    chardata
    *           The XML text to write for the given element.
    * @param    uri
    *           The namespace URI of the given element.
    * @param    handle
    *           The handle of the SAX-Attribute object to be added into XML document.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical writeDataElement(Text name, Text chardata, String uri, handle handle);

   /**
    * Creates complete XML node with a given SAX-Writer object.
    * 
    * @param    name
    *           The name of the element.
    * @param    chardata
    *           The XML text to write for the given element.
    * @param    uri
    *           The namespace URI of the given element.
    * @param    handle
    *           The handle of the SAX-Attribute object to be added into XML document.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical writeDataElement(String name, Text chardata, Text uri, handle handle);

   /**
    * Creates complete XML node with a given SAX-Writer object.
    * 
    * @param    name
    *           The name of the element.
    * @param    chardata
    *           The XML text to write for the given element.
    * @param    uri
    *           The namespace URI of the given element.
    * @param    handle
    *           The handle of the SAX-Attribute object to be added into XML document.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical writeDataElement(Text name, String chardata, Text uri, handle handle);

   /**
    * Creates complete XML node with a given SAX-Writer object.
    * 
    * @param    name
    *           The name of the element.
    * @param    chardata
    *           The XML text to write for the given element.
    * @param    uri
    *           The namespace URI of the given element.
    * @param    handle
    *           The handle of the SAX-Attribute object to be added into XML document.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical writeDataElement(String name, String chardata, Text uri, handle handle);

   /**
    * Creates complete XML node with a given SAX-Writer object.
    * 
    * @param    name
    *           The name of the element.
    * @param    chardata
    *           The XML text to write for the given element.
    * @param    uri
    *           The namespace URI of the given element.
    * @param    handle
    *           The handle of the SAX-Attribute object to be added into XML document.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical writeDataElement(String name, Text chardata, String uri, handle handle);

   /**
    * Creates complete XML node with a given SAX-Writer object.
    * 
    * @param    name
    *           The name of the element.
    * @param    chardata
    *           The XML text to write for the given element.
    * @param    uri
    *           The namespace URI of the given element.
    * @param    handle
    *           The handle of the SAX-Attribute object to be added into XML document.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical writeDataElement(Text name, String chardata, String uri, handle handle);

   /**
    * Creates complete XML node with a given SAX-Writer object.
    * 
    * @param    name
    *           The name of the element.
    * @param    chardata
    *           The XML text to write for the given element.
    * @param    uri
    *           The namespace URI of the given element.
    * @param    handle
    *           The handle of the SAX-Attribute object to be added into XML document.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical writeDataElement(String name, String chardata, String uri,
                                   handle handle);

   /**
    * Creates complete XML node with a given SAX-Writer object.
    * 
    * @param    name
    *           The name of the element.
    * @param    chardata
    *           The XML text to write for the given element.
    * @param    uri
    *           The namespace URI of the given element.
    * @param    handle
    *           The handle of the SAX-Attribute object to be added into XML document.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical writeDataElement(Text name, Text chardata, Text uri,
                                   handle handle);

   /**
    * Adds character data to the XML document crating by SAX-Writer object.
    * 
    * @param    chardata
    *           The character or longchar data to be written.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical writeFragment(Text chardata);

   /**
    * Adds character data to the XML document crating by SAX-Writer object.
    * 
    * @param    chardata
    *           The character or longchar data to be written.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical writeFragment(String chardata);

   /**
    * Adds character data to the XML document crating by SAX-Writer object.
    * 
    * @param    xhandle
    *           The valid X-Nodereference handle containing XML text.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical writeFragment(handle xhandle);
   
   /**
    * Closes the open stream and resets the SAX-Writer object to the default state.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical reset();

   /**
    * Adds comment to the XML document crating by SAX-Writer object.
    * 
    * @param    comment
    *           The character or longchar text comment to be written.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical writeComment(Text comment);

   /**
    * Adds comment to the XML document crating by SAX-Writer object.
    * 
    * @param    comment
    *           The character or longchar text comment to be written.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical writeComment(String comment);
   
   /**
    * Creates an empty node in a given SAX-Writer object.
    * 
    * @param    name
    *           The character or longchar element name to be written.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical writeEmptyElement(Text name);

   /**
    * Creates an empty node in a given SAX-Writer object.
    * 
    * @param    name
    *           The character or longchar element name to be written.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical writeEmptyElement(String name);

   /**
    * Creates an empty node in a given SAX-Writer object.
    * 
    * @param    name
    *           The character or longchar element name to be written.
    * @param    uri
    *           The character or longchar element URI to be written.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical writeEmptyElement(Text name, Text uri);

   /**
    * Creates an empty node in a given SAX-Writer object.
    * 
    * @param    name
    *           The character or longchar element name to be written.
    * @param    uri
    *           The character or longchar element URI to be written.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical writeEmptyElement(String name, Text uri);

   /**
    * Creates an empty node in a given SAX-Writer object.
    * 
    * @param    name
    *           The character or longchar element name to be written.
    * @param    uri
    *           The character or longchar element URI to be written.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical writeEmptyElement(Text name, String uri);

   /**
    * Creates an empty node in a given SAX-Writer object.
    * 
    * @param    name
    *           The character or longchar element name to be written.
    * @param    uri
    *           The character or longchar element URI to be written.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical writeEmptyElement(String name, String uri);

   /**
    * Creates an empty node in a given SAX-Writer object.
    * 
    * @param    name
    *           The character or longchar element name to be written.
    * @param    attr
    *           The handle of the SAX-Attributes object to be used.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical writeEmptyElement(Text name, handle attr);

   /**
    * Creates an empty node in a given SAX-Writer object.
    * 
    * @param    name
    *           The character or longchar element name to be written.
    * @param    attr
    *           The handle of the SAX-Attributes object to be used.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical writeEmptyElement(String name, handle attr);

   /**
    * Creates an empty node in a given SAX-Writer object.
    * 
    * @param    name
    *           The character or longchar element name to be written.
    * @param    uri
    *           The character or longchar element URI to be written.
    * @param    attr
    *           The handle of the SAX-Attributes object to be used.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical writeEmptyElement(String name, Text uri, handle attr);

   /**
    * Creates an empty node in a given SAX-Writer object.
    * 
    * @param    name
    *           The character or longchar element name to be written.
    * @param    uri
    *           The character or longchar element URI to be written.
    * @param    attr
    *           The handle of the SAX-Attributes object to be used.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical writeEmptyElement(Text name, String uri, handle attr);

   /**
    * Creates an empty node in a given SAX-Writer object.
    * 
    * @param    name
    *           The character or longchar element name to be written.
    * @param    uri
    *           The character or longchar element URI to be written.
    * @param    attr
    *           The handle of the SAX-Attributes object to be used.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical writeEmptyElement(String name, String uri, handle attr);

   /**
    * Creates an empty node in a given SAX-Writer object.
    * 
    * @param    name
    *           The character or longchar element name to be written.
    * @param    uri
    *           The character or longchar element URI to be written.
    * @param    attr
    *           The handle of the SAX-Attributes object to be used.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical writeEmptyElement(Text name, Text uri, handle attr);

   /**
    * Adds an entity referenc to the XML document associated with given SAX-Writer object.
    * 
    * @param    value
    *           The character or longchar value to be written.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical writeEntityRef(Text value);

   /**
    * Adds an entity referenc to the XML document associated with given SAX-Writer object.
    * 
    * @param    value
    *           The character or longchar value to be written.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical writeEntityRef(String value);

   /**
    * Adds an external DTD reference to the XML document associated with a SAX-Writer object.
    * 
    * @param    name
    *           The character or longchar name of the document root node.
    * @param    systemId
    *           The character or longchar system ID of the DTD.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical writeExternalDtd(Text name, Text systemId);

   /**
    * Adds an external DTD reference to the XML document associated with a SAX-Writer object.
    * 
    * @param    name
    *           The character or longchar name of the document root node.
    * @param    systemId
    *           The character or longchar system ID of the DTD.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical writeExternalDtd(String name, Text systemId);

   /**
    * Adds an external DTD reference to the XML document associated with a SAX-Writer object.
    * 
    * @param    name
    *           The character or longchar name of the document root node.
    * @param    systemId
    *           The character or longchar system ID of the DTD.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical writeExternalDtd(Text name, String systemId);

   /**
    * Adds an external DTD reference to the XML document associated with a SAX-Writer object.
    * 
    * @param    name
    *           The character or longchar name of the document root node.
    * @param    systemId
    *           The character or longchar system ID of the DTD.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical writeExternalDtd(String name, String systemId);

   /**
    * Adds an external DTD reference to the XML document associated with a SAX-Writer object.
    * 
    * @param    name
    *           The character or longchar name of the document root node.
    * @param    systemId
    *           The character or longchar system ID of the DTD.
    * @param    publicId
    *           The character or longchar public ID of the DTD.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical writeExternalDtd(String name, Text systemId, Text publicId);

   /**
    * Adds an external DTD reference to the XML document associated with a SAX-Writer object.
    * 
    * @param    name
    *           The character or longchar name of the document root node.
    * @param    systemId
    *           The character or longchar system ID of the DTD.
    * @param    publicId
    *           The character or longchar public ID of the DTD.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical writeExternalDtd(Text name, String systemId, Text publicId);

   /**
    * Adds an external DTD reference to the XML document associated with a SAX-Writer object.
    * 
    * @param    name
    *           The character or longchar name of the document root node.
    * @param    systemId
    *           The character or longchar system ID of the DTD.
    * @param    publicId
    *           The character or longchar public ID of the DTD.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical writeExternalDtd(String name, String systemId, Text publicId);

   /**
    * Adds an external DTD reference to the XML document associated with a SAX-Writer object.
    * 
    * @param    name
    *           The character or longchar name of the document root node.
    * @param    systemId
    *           The character or longchar system ID of the DTD.
    * @param    publicId
    *           The character or longchar public ID of the DTD.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical writeExternalDtd(Text name, Text systemId, String publicId);

   /**
    * Adds an external DTD reference to the XML document associated with a SAX-Writer object.
    * 
    * @param    name
    *           The character or longchar name of the document root node.
    * @param    systemId
    *           The character or longchar system ID of the DTD.
    * @param    publicId
    *           The character or longchar public ID of the DTD.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical writeExternalDtd(String name, Text systemId, String publicId);

   /**
    * Adds an external DTD reference to the XML document associated with a SAX-Writer object.
    * 
    * @param    name
    *           The character or longchar name of the document root node.
    * @param    systemId
    *           The character or longchar system ID of the DTD.
    * @param    publicId
    *           The character or longchar public ID of the DTD.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical writeExternalDtd(Text name, String systemId, String publicId);

   /**
    * Adds an external DTD reference to the XML document associated with a SAX-Writer object.
    * 
    * @param    name
    *           The character or longchar name of the document root node.
    * @param    systemId
    *           The character or longchar system ID of the DTD.
    * @param    publicId
    *           The character or longchar public ID of the DTD.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical writeExternalDtd(String name, String systemId, String publicId);

   /**
    * Adds an external DTD reference to the XML document associated with a SAX-Writer object.
    * 
    * @param    name
    *           The character or longchar name of the document root node.
    * @param    systemId
    *           The character or longchar system ID of the DTD.
    * @param    publicId
    *           The character or longchar public ID of the DTD.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical writeExternalDtd(Text name, Text systemId, Text publicId);

   /**
    * Creates a processing instruction node in XML document for given SAX-Writer object.
    * 
    * @param    target
    *           The character or longchar target of the processing instruction.
    * @param    data
    *           The character or longchar data associated with the processing instruction.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical writeProcessingInstruction(String target, Text data);

   /**
    * Creates a processing instruction node in XML document for given SAX-Writer object.
    * 
    * @param    target
    *           The character or longchar target of the processing instruction.
    * @param    data
    *           The character or longchar data associated with the processing instruction.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical writeProcessingInstruction(Text target, String data);

   /**
    * Creates a processing instruction node in XML document for given SAX-Writer object.
    * 
    * @param    target
    *           The character or longchar target of the processing instruction.
    * @param    data
    *           The character or longchar data associated with the processing instruction.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical writeProcessingInstruction(String target, String data);

   /**
    * Creates a processing instruction node in XML document for given SAX-Writer object.
    * 
    * @param    target
    *           The character or longchar target of the processing instruction.
    * @param    data
    *           The character or longchar data associated with the processing instruction.
    * 
    * @return   The <code>true</code> in case of success <code>false</code> otherwise.
    */
   public logical writeProcessingInstruction(Text target, Text data);
}
